"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const http_server_1 = require("./services/http-server");
const data_service_1 = require("./services/data-service");
const websocket_service_1 = require("./services/websocket-service");
const Debug = require("debug");
let debug = Debug('kamand');
class Server {
    run(host = '0.0.0.0', port = 8050, logger = true, origin = true, socketHost = '0.0.0.0', socketPort = 8040) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('starting services');
            this.dataService = new data_service_1.DataService();
            yield this.dataService.connect();
            this.httpServer = new http_server_1.HttpServer(this.dataService, host, port, logger, origin);
            this.httpServer.start();
            this.webSocketService = new websocket_service_1.WebSocketService(this, socketHost, socketPort);
            this.webSocketService.start();
        });
    }
    getDataService() {
        return this.dataService;
    }
    getHttpServer() {
        return this.httpServer;
    }
    getWebSocketService() {
        return this.webSocketService;
    }
    registerQueryBuilder(queryBuilders) {
        this.dataService.registerQueryBuilder(queryBuilders);
    }
    registerModel(models) {
        this.httpServer.registerModelRoutes(models);
        this.dataService.registerModelActions(models);
        models.forEach(model => model.setServer(this));
    }
    regsiterEventListener(listener) {
        listener.setServer(this);
        this.webSocketService.registerListener(listener);
        this.dataService.registerModelActions([listener]);
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            debug('stopping services');
            yield this.httpServer.stop();
            yield this.dataService.stop();
            yield this.webSocketService.stop();
        });
    }
}
exports.Server = Server;
//# sourceMappingURL=server.js.map