"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterateOnQuery = exports.DataService = void 0;
const pg_1 = require("pg");
const Debug = require("debug");
const http_errors_1 = require("http-errors");
const uuid_1 = require("uuid");
let debug = Debug('kamand');
const parseDate = function (val) {
    //K1: I had a problem, the default behaviour case the reading to go back/forward
    return val + 'T00:00:00.000Z';
};
pg_1.types.setTypeParser(pg_1.types.builtins.DATE, parseDate);
const parseTimestamp = function (val) {
    //K1: I normally do not use time on server, so let's not waist time and energy for that
    return val;
};
pg_1.types.setTypeParser(pg_1.types.builtins.TIMESTAMP, parseTimestamp);
pg_1.types.setTypeParser(pg_1.types.builtins.TIMESTAMPTZ, parseTimestamp);
class DataService {
    constructor() {
        this.config = {
            host: process.env.DB_HOST,
            port: parseInt(process.env.DB_PORT || '5432'),
            password: process.env.DB_PASS,
            user: process.env.DB_USER,
            database: process.env.DB_DATABASE,
        };
        this.poolConfig = Object.assign(Object.assign({}, this.config), { max: 10, idleTimeoutMillis: 5 * 60 * 1000, connectionTimeoutMillis: 10 * 1000 });
        this.queryBuilders = new Map();
        this.actions = new Map();
        this.notificationListeners = new Map();
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.dataPool = new pg_1.Pool(this.poolConfig);
            this.dataPool.on('error', (error) => {
                debug(`pg unhandled error ${error}`);
            });
            debug(`connected to pg ${this.poolConfig.host}`);
            this.notificationClient = new pg_1.Client(this.config);
            yield this.notificationClient.connect();
            this.notificationClient.on('notification', (msg) => __awaiter(this, void 0, void 0, function* () {
                const nls = this.notificationListeners.get(msg.channel);
                if (nls) {
                    nls.forEach(nl => nl.notify(msg.payload));
                }
            }));
            debug(`connected to pg ${this.poolConfig.host}`);
        });
    }
    registerNotificationListener(notificationListener) {
        const { channel } = notificationListener;
        const nls = this.notificationListeners.get(channel);
        if (!nls) {
            this.notificationListeners.set(channel, [notificationListener]);
            this.notificationClient.query(`LISTEN ${channel}`);
        }
        else {
            nls.push(notificationListener);
        }
    }
    registerQueryBuilder(queryBuilders) {
        queryBuilders.forEach(queryBuilder => {
            this.queryBuilders.set(queryBuilder.query, queryBuilder);
        });
    }
    query(query, queryParams, user) {
        return __awaiter(this, void 0, void 0, function* () {
            let client;
            try {
                // debug(JSON.stringify(this.queryBuilders, null, 2))
                const queryBuilder = this.queryBuilders.get(query);
                if (!queryBuilder) {
                    throw new Error(`query ${query} not found!`);
                }
                if (!queryBuilder.public) {
                    if (!user) {
                        throw new http_errors_1.Unauthorized(`no user defined`);
                    }
                    if (!queryBuilder.authorize) {
                        throw new http_errors_1.Unauthorized(`no authorize function defined!`);
                    }
                    if (!queryBuilder.authorize(user)) {
                        throw new http_errors_1.Unauthorized(`user has no access`);
                    }
                }
                client = yield this.dataPool.connect();
                // console.time('app_name')
                yield client.query(`set application_name = 'query_${query}';`);
                // it has 4 ms over head
                // console.timeEnd('app_name')
                const result = yield client.query(queryBuilder.createQueryConfig(queryParams, user));
                client.release();
                return result.rows;
            }
            catch (error) {
                if (client) {
                    try {
                        client.release(error);
                    }
                    catch (error) { }
                }
                throw error;
            }
        });
    }
    giveDbClient() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.dataPool.connect();
        });
    }
    stop() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            debug('stopping data service');
            yield ((_a = this.dataPool) === null || _a === void 0 ? void 0 : _a.end());
            yield ((_b = this.notificationClient) === null || _b === void 0 ? void 0 : _b.end());
        });
    }
    registerModelActions(models) {
        models.forEach(model => {
            const modelAddress = model.address();
            const actions = model.actions();
            actions.forEach(action => {
                this.actions.set(modelAddress + action.address(), action);
            });
        });
    }
    act(address, actionParams, user) {
        return __awaiter(this, void 0, void 0, function* () {
            let client;
            try {
                // debug(JSON.stringify(this.queryBuilders, null, 2))
                const action = this.actions.get(address);
                if (!action) {
                    throw new http_errors_1.NotFound(`action ${address} not found!`);
                }
                if (!action.public) {
                    if (!user) {
                        // if you encounter this error, the route that is calling this action is defined as public
                        throw new http_errors_1.Unauthorized(`no user defined but the action is private`);
                    }
                }
                client = yield this.dataPool.connect();
                // it has 4ms overhead
                yield client.query(`set application_name = 'action_${address}';`);
                yield client.query('begin');
                const result = yield action.act(client, actionParams, user);
                yield client.query('commit');
                client.release();
                return result;
            }
            catch (error) {
                if (client) {
                    try {
                        yield client.query('rollback');
                    }
                    catch (error) { }
                    try {
                        client.release(error);
                    }
                    catch (error) { }
                }
                throw error;
            }
        });
    }
    actInMyTransaction(client, address, actionParams, user) {
        return __awaiter(this, void 0, void 0, function* () {
            // debug(JSON.stringify(this.queryBuilders, null, 2))
            const action = this.actions.get(address);
            if (!action) {
                throw new http_errors_1.NotFound(`action ${address} not found!`);
            }
            if (!action.public) {
                if (!user) {
                    // if you encounter this error, the route that is calling this action is defined as public
                    throw new http_errors_1.Unauthorized(`no user defined but the action is private`);
                }
            }
            const result = yield action.act(client, actionParams, user);
            return result;
        });
    }
}
exports.DataService = DataService;
function iterateOnQuery(client, query, chunkSize, arrayMode = true) {
    return __asyncGenerator(this, arguments, function* iterateOnQuery_1() {
        const name = uuid_1.v4();
        try {
            yield __await(client.query(`DECLARE "${name}" NO SCROLL CURSOR FOR ${query.text}`, query.values));
            while (true) {
                const record = yield __await(client.query({ text: `FETCH ${chunkSize} "${name}"`, rowMode: arrayMode ? 'array' : undefined }));
                if (record.rows.length === 0) {
                    break;
                }
                yield yield __await(record);
            }
        }
        finally {
            try {
                yield __await(client.query(`CLOSE "${name}"`));
            }
            catch (err) {
                debug(err);
            }
        }
    });
}
exports.iterateOnQuery = iterateOnQuery;
//# sourceMappingURL=data-service.js.map