"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpServer = void 0;
const fastify = require("fastify");
require("fastify-cors");
require("fastify-jwt");
require("fastify-file-upload");
const fastifyCors = require("fastify-cors");
const fastifyJwt = require("fastify-jwt");
const fastifyFileUpload = require("fastify-file-upload");
const http_errors_1 = require("http-errors");
const Debug = require("debug");
let debug = Debug('kamand');
class HttpServer {
    constructor(dataService, host, port, logger, origin) {
        this.dataService = dataService;
        this.host = host;
        this.port = port;
        this.logger = logger;
        this.origin = origin;
    }
    start() {
        this.fastifyServer = fastify({
            bodyLimit: 100 * 1024,
            logger: this.logger,
        });
        this.fastifyServer.register(fastifyCors, {
            origin: this.origin,
        });
        this.fastifyServer.register(fastifyJwt, {
            secret: process.env.JWT_SECRET_KEY,
        });
        this.fastifyServer.register(fastifyFileUpload, {
            limits: { fileSize: 2 * 1024 * 1024 },
            abortOnLimit: true,
        });
        this.fastifyServer.decorate("authenticate", function (request, reply) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    yield request.jwtVerify();
                }
                catch (err) {
                    reply.send(err);
                }
            });
        });
        this.fastifyServer.get('/data/:queryData', {}, (req, reply) => __awaiter(this, void 0, void 0, function* () {
            const query = req.params.queryData;
            const queryParams = req.query;
            try {
                const result = yield this.dataService.query(query, queryParams);
                reply.type('application/json').code(200);
                return result;
            }
            catch (error) {
                if (error.statusCode) {
                    reply.send(error);
                }
                else {
                    console.error(`error while processing data with query : ${query}\n params: ${JSON.stringify(queryParams, null, 2)}\n${error}`);
                    reply.send(new http_errors_1.InternalServerError());
                }
            }
        }));
        this.fastifyServer.get('/private/data/:queryData', {
            preValidation: [this.fastifyServer.authenticate]
        }, (req, reply) => __awaiter(this, void 0, void 0, function* () {
            const query = req.params.queryData;
            const queryParams = req.query;
            try {
                const result = yield this.dataService.query(query, queryParams, req.user);
                reply.type('application/json').code(200);
                return result;
            }
            catch (error) {
                if (error.statusCode) {
                    reply.send(error);
                }
                else {
                    console.error(`error while processing private/data with query : ${query}\n params: ${JSON.stringify(queryParams, null, 2)}\n${error}`);
                    reply.send(new http_errors_1.InternalServerError());
                }
            }
        }));
        this.fastifyServer.listen(this.port, this.host, (err, address) => {
            if (err)
                throw err;
            debug(`listen on ${address}`);
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            debug('stopping http server');
            yield this.fastifyServer.close();
        });
    }
    sign(payload, options) {
        return this.fastifyServer.jwt.sign(payload, options);
    }
    verify(token, options) {
        return this.fastifyServer.jwt.verify(token, options);
    }
    registerModelRoutes(models) {
        models.map(m => {
            const address = m.address();
            const routes = m.routes();
            routes.forEach(route => {
                route.url = address + route.url;
                if (!route.public) {
                    route.preValidation = this.fastifyServer.authenticate;
                }
                this.fastifyServer.route(route);
            });
        });
    }
}
exports.HttpServer = HttpServer;
//# sourceMappingURL=http-server.js.map