"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createError = exports.throwAdvancedError = exports.throwError = exports.isValidNationalID = exports.isValidNumber = exports.isValidTimestamp = exports.allCharsAreDigits = exports.isValidPersianAlphabetFormat = exports.isValidMobileFormat = exports.isValidEmailFormat = void 0;
const http_errors_1 = require("http-errors");
const emailFormat = /\S+@\S+\.\S+/;
const mobileFormat = /^(09)\d{9}$/;
const persianAlphabetFormat = /^([\u0600-\u06FF|\s|\u200C])+$/;
const allDigitFormat = /^\d+$/;
function isValidEmailFormat(email) {
    return emailFormat.test(email);
}
exports.isValidEmailFormat = isValidEmailFormat;
function isValidMobileFormat(email) {
    return mobileFormat.test(email);
}
exports.isValidMobileFormat = isValidMobileFormat;
function isValidPersianAlphabetFormat(text) {
    return persianAlphabetFormat.test(text);
}
exports.isValidPersianAlphabetFormat = isValidPersianAlphabetFormat;
function allCharsAreDigits(string) {
    return allDigitFormat.test(string);
}
exports.allCharsAreDigits = allCharsAreDigits;
function isValidTimestamp(value) {
    if (value instanceof Date) {
        return true;
    }
    if (typeof value === 'string') {
        const d = Date.parse(value);
        return !isNaN(d);
    }
    return false;
}
exports.isValidTimestamp = isValidTimestamp;
function isValidNumber(value) {
    if (typeof value === 'number') {
        return true;
    }
    if (typeof value === 'string') {
        const d = Number(value);
        return !isNaN(d);
        // return allDigitFormat.test(value);
    }
    return false;
}
exports.isValidNumber = isValidNumber;
function isValidNationalID(value) {
    if (value === "")
        return true;
    if (!allCharsAreDigits(value))
        return false;
    if (value.length !== 10)
        return false;
    let sum = 0;
    let controlDigit = parseInt(value[9], 10);
    for (let i = 8; i >= 0; i--) {
        sum += parseInt(value[i], 10) * (10 - i);
    }
    let reminder = sum % 11;
    if (reminder >= 2)
        return 11 - reminder === controlDigit;
    else
        return reminder === controlDigit;
}
exports.isValidNationalID = isValidNationalID;
function throwError(property, errorCode, errorText, fieldTranslation) {
    throwAdvancedError(property, errorCode, errorText, { field: fieldTranslation });
}
exports.throwError = throwError;
function throwAdvancedError(property, errorCode, errorText, params) {
    const msg = {
        codes: { [property]: [{ code: errorCode, params }] },
        [property]: [errorText],
    };
    const error = new http_errors_1.ExpectationFailed(JSON.stringify(msg));
    throw error;
}
exports.throwAdvancedError = throwAdvancedError;
function createError(path, errorCode, errorText, params) {
    const err = {
        path,
        message: errorText,
        code: errorCode,
        params,
    };
    return err;
}
exports.createError = createError;
//# sourceMappingURL=value-validators.js.map