"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketService = void 0;
const SocketIOServer = require("socket.io");
// import { RaceOrganiser } from './race/race-organiser';
// import { RaceController } from './race/race-controller';
const http = require("http");
const Debug = require("debug");
const debug = Debug('kamand:websocket');
class WebSocketService {
    constructor(server, host, port) {
        this.server = server;
        this.host = host;
        this.port = port;
        this.listeners = [];
        this.connection = (socket) => {
            socket.on('disconnect', (reason) => this.disconnected(socket, reason));
            socket.on('error', (error) => this.error(socket, error));
            socket.on('authorize', (data) => this.authorize(socket, data));
            this.listeners.forEach(listener => {
                socket.on(listener.query, (payload) => {
                    listener.listener(socket, payload);
                });
            });
        };
        this.authorize = (socket, data) => {
            try {
                const payload = this.server.getHttpServer().verify(data);
                socket.user = payload;
            }
            catch (error) {
                debug(`error while verifying token with ${error}\n the token was: \n ${data}`);
            }
        };
        this.disconnected = (socket, reason) => {
            // debug(`socket ${socket.id} disconnected with reason ${reason}`);
            this.removeForFutureUpdateBySocket(socket);
        };
        this.error = (socket, error) => {
            debug(`socket ${socket.id} encountered error: ${error === null || error === void 0 ? void 0 : error.toString()}`);
        };
        this.options = {
            path: '/kamand-io',
            serveClient: false,
            cookie: false,
        };
        this.io = SocketIOServer(this.options);
        this.httpServer = http.createServer();
        this.futureUpdates = new Map();
        this.futureUpdateKeys = new Map();
    }
    use(fn) {
        return this.io.use(fn);
    }
    // route() {
    //   const namespace = '/race';
    //   const nsp = namespace ? this.io.of(namespace) : this.io;
    //   nsp.on('connect', async socket => {
    //     debug(
    //       'Websocket connected: id=%s namespace=%s',
    //       socket.id,
    //       socket.nsp.name,
    //     );
    //     new RaceController(socket, this.raceOrganiser);
    //   });
    // }
    registerListener(listener) {
        this.listeners.push(listener);
    }
    getIo() {
        return this.io;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            debug(`websocket starting ${this.host} ${this.port}`);
            this.httpServer.listen(this.port, this.host);
            this.io.attach(this.httpServer, this.options);
            this.io.on('connection', this.connection);
            debug('websocket started');
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            this.httpServer.close();
            const close = new Promise((resolve, reject) => {
                this.io.close(() => {
                    resolve();
                });
            });
            yield close;
        });
    }
    addForFutureUpdate(key, socket, payload) {
        const futureUpdates = this.futureUpdates.get(key);
        if (!futureUpdates) {
            this.futureUpdates.set(key, [{ socket, payload }]);
            this.addFutureUpdateKey(socket.id, key);
        }
        else {
            const index = futureUpdates.findIndex(futureUpdate => futureUpdate.socket === socket);
            if (index === -1) {
                futureUpdates.push({ socket, payload });
                this.addFutureUpdateKey(socket.id, key);
            }
        }
    }
    removeForFutureUpdate(key, socket) {
        const futureUpdates = this.futureUpdates.get(key);
        if (futureUpdates) {
            const index = futureUpdates.findIndex(({ socket: s }) => s === socket);
            if (index !== -1) {
                futureUpdates.splice(index);
            }
        }
        this.removeFutureUpdateKey(socket.id, key);
    }
    findForFutureUpdate(key) {
        return this.futureUpdates.get(key);
    }
    addFutureUpdateKey(socketId, key) {
        const keys = this.futureUpdateKeys.get(socketId);
        if (!keys) {
            this.futureUpdateKeys.set(socketId, [key]);
        }
        else {
            keys.push(key);
        }
    }
    removeFutureUpdateKey(socketId, key) {
        const keys = this.futureUpdateKeys.get(socketId);
        if (keys) {
            const index = keys.findIndex(k => k === key);
            if (index !== -1) {
                keys.splice(index);
            }
        }
    }
    removeForFutureUpdateBySocket(socket) {
        const keys = this.futureUpdateKeys.get(socket.id);
        if (keys) {
            keys.forEach(key => {
                const futureUpdates = this.futureUpdates.get(key);
                const index = futureUpdates.findIndex(({ socket: s }) => s === socket);
                if (index) {
                    futureUpdates.splice(index);
                }
            });
            this.futureUpdateKeys.delete(socket.id);
        }
    }
}
exports.WebSocketService = WebSocketService;
//# sourceMappingURL=websocket-service.js.map