"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.camelCaseObject = exports.setNestedObjectValues = exports.setIn = exports.getIn = exports.clone = exports.toPath = exports.hasNonEmptyValue = exports.isPromise = exports.isNaN = exports.isString = exports.isInteger = exports.isObject = exports.isFunction = exports.sleep = exports.pause = void 0;
function pause(timout) {
    // tslint:disable-next-line:no-shadowed-variable
    return new Promise((resolve) => {
        setTimeout(() => resolve(), timout);
    });
}
exports.pause = pause;
exports.sleep = pause;
/** @private is the given object a Function? */
exports.isFunction = (obj) => typeof obj === 'function';
/** @private is the given object an Object? */
exports.isObject = (obj) => obj !== null && typeof obj === 'object';
/** @private is the given object an integer? */
exports.isInteger = (obj) => String(Math.floor(Number(obj))) === obj;
/** @private is the given object a string? */
exports.isString = (obj) => Object.prototype.toString.call(obj) === '[object String]';
/** @private is the given object a NaN? */
// eslint-disable-next-line no-self-compare
exports.isNaN = (obj) => obj !== obj;
/** @private is the given object/value a promise? */
exports.isPromise = (value) => exports.isObject(value) && exports.isFunction(value.then);
exports.hasNonEmptyValue = (obj) => {
    if (exports.isObject(obj)) {
        if (Array.isArray(obj)) {
            for (const v of obj) {
                if (exports.hasNonEmptyValue(v))
                    return true;
            }
        }
        else {
            for (const key of Object.keys(obj)) {
                if (exports.hasNonEmptyValue(obj[key]))
                    return true;
            }
        }
    }
    else {
        if (obj) {
            return true;
        }
    }
    return false;
};
exports.toPath = (key) => {
    const path = [];
    const parts = key.split('.');
    parts.forEach(part => {
        const i1 = part.indexOf('[');
        if (i1 >= 0) {
            path.push(part.substring(0, i1));
            const i2 = part.indexOf(']');
            if (i2 >= 0) {
                path.push(part.substring(i1 + 1, i2));
            }
            else {
                //error but ignore
            }
        }
        else {
            path.push(part);
        }
    });
    return path;
    /*
  _.toPath('a.b.c');
  // => ['a', 'b', 'c']
   
  _.toPath('a[0].b.c');
  // => ['a', '0', 'b', 'c']
     */
};
exports.clone = (obj) => {
    if (Array.isArray(obj)) {
        return [...obj];
    }
    else if (exports.isObject(obj)) {
        return Object.assign({}, obj);
    }
    else {
        return obj;
    }
};
function getIn(obj, key, def, p = 0) {
    const path = Array.isArray(key) ? key : exports.toPath(key);
    while (obj && p < path.length) {
        obj = obj[path[p++]];
    }
    return obj === undefined ? def : obj;
}
exports.getIn = getIn;
function setIn(obj, path, value) {
    let res = exports.clone(obj); // this keeps inheritance when obj is a class
    let resVal = res;
    let i = 0;
    let pathArray = exports.toPath(path);
    for (; i < pathArray.length - 1; i++) {
        const currentPath = pathArray[i];
        let currentObj = getIn(obj, pathArray.slice(0, i + 1));
        if (currentObj && (exports.isObject(currentObj) || Array.isArray(currentObj))) {
            resVal = resVal[currentPath] = exports.clone(currentObj);
        }
        else {
            const nextPath = pathArray[i + 1];
            resVal = resVal[currentPath] =
                exports.isInteger(nextPath) && Number(nextPath) >= 0 ? [] : {};
        }
    }
    // Return original object if new value is the same as current
    if ((i === 0 ? obj : resVal)[pathArray[i]] === value) {
        return obj;
    }
    if (value === undefined) {
        delete resVal[pathArray[i]];
    }
    else {
        resVal[pathArray[i]] = value;
    }
    // If the path array has a single element, the loop did not run.
    // Deleting on `resVal` had no effect in this scenario, so we delete on the result instead.
    if (i === 0 && value === undefined) {
        delete res[pathArray[i]];
    }
    return res;
}
exports.setIn = setIn;
/**
 * Recursively a set the same value for all keys and arrays nested object, cloning
 * @param object
 * @param value
 * @param visited
 * @param response
 */
function setNestedObjectValues(object, value, visited = new WeakMap(), response = {}) {
    for (let k of Object.keys(object)) {
        const val = object[k];
        if (exports.isObject(val)) {
            if (!visited.get(val)) {
                visited.set(val, true);
                // In order to keep array values consistent for both dot path  and
                // bracket syntax, we need to check if this is an array so that
                // this will output  { friends: [true] } and not { friends: { "0": true } }
                response[k] = Array.isArray(val) ? [] : {};
                setNestedObjectValues(val, value, visited, response[k]);
            }
        }
        else {
            response[k] = value;
        }
    }
    return response;
}
exports.setNestedObjectValues = setNestedObjectValues;
const change_case_1 = require("change-case");
function camelCaseObject(data) {
    if (Array.isArray(data)) {
        const newData = data.map(d => {
            if (d && typeof d === "object") {
                return camelCaseObject(d);
            }
            else {
                return d;
            }
        });
        return newData;
    }
    else {
        const newData = {};
        Object.keys(data).forEach(function (key) {
            if (data[key] && typeof data[key] === "object") {
                newData[change_case_1.camelCase(key)] = camelCaseObject(data[key]);
            }
            else {
                newData[change_case_1.camelCase(key)] = data[key];
            }
        });
        return newData;
    }
}
exports.camelCaseObject = camelCaseObject;
//# sourceMappingURL=generalUtils.js.map